import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
import java.util.Collections;

/**
 * The world that everything lives in.
 * 
 * @author Michael Berry
 * @version 17/07/08
 */
public class BounceWorld extends World
{

    /** The number of lives the player starts with */
    public static final int STARTLIVES = 5;
    /** The number of levels */
    public static final int MAXLEVEL = 11;
    /** The time to wait between each level */
    public static final int DELAY = 30;
    
    private Text levelText, livesText, coinsText;
    private HighScore highscore;
    private Ball ball;
    private int level, lives, coins;
    private MuteSwitch muteswitch;

    /**
     * Create a new world.
     */
    public BounceWorld()
    {    
        super(700,500,1);
        Greenfoot.setSimulationSpeed(50);
        setPaintOrder(Text.class, BigText.class);
        
        level = 1;
        levelText = new Text();
        addObject(levelText, 70,50);
        
        lives = STARTLIVES;
        livesText = new Text();
        addObject(livesText, 70,70);
        livesText.setText("Lives: " + lives);
        
        coins = 0;
        coinsText = new Text();
        addObject(coinsText, 70,90);
        coinsText.setText("Coins: " + coins);
        
        highscore = new HighScore("bounce");
        
        muteswitch = new MuteSwitch();
        addObject(muteswitch, getWidth()-20, getHeight()-20);
       
        createLevel(true);
    }
    
    /**
     * Plays a sound if the mute switch is off.
     */
    public void playSound(String s)
    {
        if(! muteswitch.isMute()) {
            Greenfoot.playSound(s);
        }
    }
    
    /**
     * Adds a coin to the player's score.
     */
    public void addCoin()
    {
        coins++;
        coinsText.setText("Coins: " + coins);
    }
    
    /**
     * Remove actors from the world.
     * @param b whether to remove stationary objects or not, true if stationary objects should be removed.
     * Stationary objects include text and mute switches.
     */
    private void removeActors(boolean b)
    {
        for(Actor actor : (List<Actor>)getObjects(null)) {
            boolean remove = true;
            if(actor instanceof Text) remove = false;
            if(actor instanceof MuteSwitch) remove = false;
            if(b) remove = true;
            if(remove) removeObject(actor);
        }      
    }
    
    /**
     * Called when ball has fallen from world
     */
    public void fallen()
    {
        lives--;
        livesText.setText("Lives: " + lives);
        if(lives==0) {
            removeActors(true);
            gameOver(false);
        }
        else {
            createLevel(false);
            playSound("sounds/lose.wav");
        }
        
    }
    
    /**
     * Called when the goal has been reached
     */
    public void won()
    {
        playSound("sounds/win.wav");
        if(level != MAXLEVEL) {
            level++;
            createLevel(true);
        }
        else {
            removeActors(true);
            gameOver(true);
        }
    }
    
    /**
     * Called when all lives have been lost
     */
    private void gameOver(boolean complete)
    {
        submitHighScore();
        for(BigText text : (List<BigText>)getObjects(BigText.class)) {
            removeObject(text);
        }
        BigText text = new BigText(false);
        addObject(text, 175,80);
        if(complete) {
            text.setText("Game Complete!");
        }
        else {
            text.setText("Game Over");
        }
        BigText htext = new BigText(false);
        addObject(htext, 170,260);
        htext.setText("High Scores");
        displayHighScores(20);
        Greenfoot.stopSimulation();
    }
    
    /**
     * Submit the score
     */
    private void submitHighScore()
    {
        highscore.addScore(coins);
    }
    
    /**
     * Display the high scores
     * @param num the number of high scores to display.
     */
    private void displayHighScores(int num)
    {
        List<Score> scores = highscore.getScores();
        Collections.sort(scores);
        Collections.reverse(scores);
        for(int i=0 ; i<num && i<scores.size() ; i++) {
            Score score = scores.get(i);
            Text text = new Text();
            addObject(text, 550, (i*23)+60);
            text.setText(score.getName() + ": " + score.getScore());
        }
    }
    
    /**
     * Setup the level. Blocks are in a different place for each level,
     * and this needs to be definied specifically.
     * @param coins whether to create the coins for each level. false if the
     * player is retaking this level because they lost a life.
     */
    private void createLevel(boolean coins)
    {
        removeActors(false);
        if(level==1) {
            int j=40;
            for(int i=20 ; i<400 ; i += 40 ) {
                j+=20;
                addObject(new NormalBlock(), i, getHeight()-j);
                if(i%60==0 && coins) addObject(new Coin(), i, getHeight()-j-4-new Coin().getHeight()/2);
            }  
            
            addObject(new Goal(), 350, 25);
            addObject((ball = new Ball()), 60, 90);
        }
        else if(level==2) {
            for(int i=20 ; i<400 ; i += 40) {
                addObject(new NormalBlock(), i, getHeight()-50);
            }
            for(int i=440 ; i<=520 ; i += 40) {
                addObject(new NormalBlock(), i, 340);
            }
            for(int i=540 ; i<=620 ; i += 40) {
                addObject(new NormalBlock(), i, 220);
            }
            for(int i=440 ; i<=520 ; i += 40) {
                addObject(new NormalBlock(), i, 100);
            }
            for(int i=540 ; i<=620 ; i += 40) {
                addObject(new NormalBlock(), i, 460);
                if(coins) addObject(new Coin(), i, 430);
            } 
            if(coins) addObject(new Coin(), 480, 310);
            if(coins) addObject(new Coin(), 580, 190);
            if(coins) addObject(new Coin(), 480, 70);  
            
            addObject(new Goal(), 350, 25);
            addObject((ball = new Ball()), 60, 90);
        }
        else if(level==3) {
            addObject(new Goal(), 110, 390);
            addObject((ball = new Ball()), 60, 90);
            addObject(new BreakBlock(1), 350, 455);
            if(coins) addObject(new Coin(), 350, 425);            
        }
        else if(level==4) {
            addObject((ball = new Ball()), 60, 90);
            int j=340;
            for(int i=400 ; i>20 ; i -= 40 ) {
                j-=20;
                addObject(new DudBlock(), i, j);
                if(i%60==0 && coins) addObject(new Coin(), i, j-30);
            }
            addObject(new BreakBlock(1), 500, 400);
            addObject(new Goal(), 660, 290);
        }
        else if(level==5) {
            addObject((ball = new Ball()), 60, 90);
            int j=340;
            for(int i=400 ; i>20 ; i -= 40 ) {
                j-=20;
                addObject(new DudBlock(), i, j);
                if(i%60==0 && coins) addObject(new Coin(), i, j-30);
            }
            addObject(new Goal(), 660, 290);
        }
        else if(level==6) {
            addObject((ball = new Ball()), 60, 200);
            for(int i=60 ; i<getWidth()-160 ; i+=100) {
                addObject(new BreakBlock(1), i, getHeight()-100);
                if(coins) addObject(new Coin(), i, getHeight()-130);                
            }
            addObject(new BreakBlock(2), getWidth()-100, getHeight()/2);
            addObject(new Goal(), getWidth()/2, 40);
        }
        else if(level==7) {
            addObject((ball = new Ball()), 60, 200);
            addObject(new NormalBlock(), 60, 400);
            if(coins) addObject(new Coin(), 60, 380);
            addObject(new MoveBlock(100, 2, MoveBlock.Direction.RIGHT), getWidth()/2, getHeight()/2);
            if(coins) addObject(new Coin(), getWidth()/2, getHeight()/2-30);
            addObject(new Goal(), getWidth()/2, 40);
        }
        else if(level==8) {
            addObject((ball = new Ball()), 60, 200);
            for(int i=60 ; i<getWidth()-160 ; i+=100) {
                addObject(new MoveBlock(40, 2, MoveBlock.Direction.UP), i, getHeight()-100);
                if(coins) addObject(new Coin(), i, getHeight()-130);                
            }
            addObject(new BreakBlock(3), getWidth()-100, getHeight()/2);
            addObject(new DudBlock(), 490, 160);
            addObject(new DudBlock(), 455, 160);            
            addObject(new MoveBlock(100, 2, MoveBlock.Direction.RIGHT), 280, 170);
            
            addObject(new Goal(), 150, 40);            
        }
        else if(level==9) {
            addObject((ball = new Ball()), 60, 200);
            for(int i=60 ; i<getWidth()-160 ; i+=100) {
                addObject(new BreakBlock(1), i, getHeight()-100);
                if(coins) addObject(new Coin(), i, getHeight()-130);                
            }
            addObject(new BreakBlock(3), getWidth()-100, getHeight()/2);
            addObject(new DudBlock(), 490, 160);
            addObject(new DudBlock(), 455, 160);            
            addObject(new MoveBlock(100, 2, MoveBlock.Direction.RIGHT), 280, 170);
            
            addObject(new Goal(), 150, 40);        
        }
        else if(level==10) {
            addObject(new Goal(), 200, 40);
            addObject(new Ball(), 50, 120);
            int j=300;
            for(int i=50 ; i<200 ; i += 40 ) {
                j+=20;
                addObject(new DudBlock(), i, j);
                if(coins) addObject(new Coin(), i, j-30);
            }
            addObject(new MoveBlock(200, 4, MoveBlock.Direction.RIGHT), 300, 400);
            addObject(new NormalBlock(), 600, 225);
            if(coins) addObject(new Coin(), 600, 195);
            addObject(new BreakBlock(2), 450, 180);
            if(coins) addObject(new Coin(), 450, 150);
            addObject(new BreakBlock(2), 330, 130);
            if(coins) addObject(new Coin(), 330, 100);
        }
        else if(level==11) {
            addObject(new Ball(), 50, 120);
            addObject(new Goal(), getWidth()-50, 120);
            for(int i=20 ; i<getWidth()-50 ; i += 40 ) {
                addObject(new DudBlock(), i, getHeight()-50);
                if(coins) addObject(new Coin(), i, getHeight()-110);
                if(coins) addObject(new Coin(), i, getHeight()-80);
            }
            for(int i=20 ; i<getWidth()-100 ; i += 40 ) {
                addObject(new BreakBlock(10), i, getHeight()/2);
                for(int j=40 ; j<getHeight()/2-30 ; j+=40) {
                    if(coins) addObject(new Coin(), i, j);
                }
            }
            addObject(new NormalBlock(), getWidth()-50, getHeight()-50);
            addObject(new MoveBlock(60, 1, MoveBlock.Direction.UP), getWidth()-20, getHeight()-100);
        }
        else {
            throw new RuntimeException("Level " + level + " doesn't exist... yet...");
        }
        
        //This stuff needs to happen on the creation of every level.

        BigText text = new BigText(true);
        addObject(text, 200,250);
        text.setText("Level " + level);
        levelText.setText("Level " + level);
        Greenfoot.delay(DELAY);
    }    
}
